#include <stdio.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <strings.h>
#include <fs.h>
#include <fs.h>
#include <inttypes.h>
     
struct traffic_st
{
	char ip[21];
	double in, out, total;
};
		
		
struct vsip_entry
{
	char *name;
	char *ip;				
};
	
	
uint32_t str2ip(char *ipstr)
{
	char *octet;
	uint32_t ip = 0;
	
	
	octet = strtok(ipstr, ".");
	if(octet != NULL)
		ip = ip | (atoi(octet)<<24);
	
	octet = strtok(NULL, ".");
	if(octet != NULL)
		ip = ip | (atoi(octet)<<16);
	
	octet = strtok(NULL, ".");
	if(octet != NULL)
		ip = ip | (atoi(octet)<<8);
	
	octet = strtok(NULL, ".");
	if(octet != NULL)
		ip = ip | atoi(octet);
	
	
	return ip;
}
	


char **parse_ips(char *name, int *num)
{
        int counter = 0;
        struct dirent *dir_info;
        char *path, *ip;
        DIR *dirst;
	char *subnet;
		
        char **output = NULL;
        *num = 0;


        path = concat("/etc/vservers/", name,"/interfaces", NULL);
        dirst = opendir(path);
        free(path);

        if(dirst != NULL)
        {

                while ((dir_info = readdir(dirst)) != NULL)
                {

                        if(strcmp(dir_info->d_name, ".") != 0 && strcmp(dir_info->d_name, "..") != 0)
                        {

                                path = concat("/etc/vservers/", name,"/interfaces/", dir_info->d_name, NULL);

                                if(dir_exist(path))
                                {
                                        free(path);
                                        path = concat("/etc/vservers/", name,"/interfaces/", dir_info->d_name, "/ip", NULL);

                                        ip = file_read(path);
                                        if(ip != NULL)
                                        {
                                                trim_string(ip);
						subnet = strstr(ip, "/");
						if(subnet != NULL)
							subnet[0] = 0;
							
                                                output = realloc(output, (counter+1) * sizeof(char **));
                                                output[counter] = ip;


                                                counter++;
                                        }
                                }

                                free(path);

                        }

                }

                *num = counter;
                closedir(dirst);
        }

        return output;
}


struct vsip_entry *vserver_iplist(void)
{
	struct vsip_entry *allips = NULL;
	char **vsips;
	int i, entrysnum, allentrys = 0;
	
	DIR *dir_ptr;
	struct dirent *dir_entry;
	char path[1024];



	if((dir_ptr = opendir("/etc/vservers/")) != NULL)
	{
		
		while((dir_entry = readdir(dir_ptr)) != NULL)
		{
			

			if(dir_entry->d_name[0] != '.')
			{
				sconcat(path, 1024, "/etc/vservers/", dir_entry->d_name, NULL);

				if(dir_exist(path))
				{
					i = 0;
					vsips = parse_ips(dir_entry->d_name, &entrysnum);
					
					while(i<entrysnum)
					{
						
						allips = realloc(allips, sizeof(struct vsip_entry)*(allentrys+2));
						allips[allentrys].name = strdup(dir_entry->d_name);
						allips[allentrys].ip = strdup(vsips[i]);
						
						allentrys++;
						i++;
					}	
					
					free_strings(vsips, entrysnum);	

				}
			}
	
		}


		closedir(dir_ptr);
	}
	
	if(allips != NULL)
	{
		allips[allentrys].name = NULL;
		allips[allentrys].ip = NULL;
	}
	
	return allips;
}

int free_iplist(struct vsip_entry *list)
{
	int i=0;
	
	if(list == NULL)
		return -1;
	
	while(list[i].name != NULL && list[i].ip != NULL)
	{
		free(list[i].name);
		free(list[i].ip);
	
		i++;
	}

	free(list);

	return 0;
}

struct traffic_st *parse_day_traffic(char *ipfm_dir, int month, int day, double *day_in, double *day_out, double *day_total, int *entry_num)
{
	FILE *logfile;
	char buf[1024+1], *ip;
	double in, out, total; 
	struct traffic_st *day_traffic = NULL;

	*day_in = 0;
	*day_out = 0;
	*day_total = 0;
	
	*entry_num = 0;

	snprintf(buf, 1024, "%s/%02d/%02d", ipfm_dir, month, day);

	if(file_exist(buf))
	{
	
		logfile = fopen(buf, "r");


		while(fgets(buf, 1024, logfile) != NULL)
		{
			
			if(buf[0] != '#')
			{
		

				sscanf(buf, "%as %lf %lf %lf", &ip, &in, &out, &total);
				
				
				day_traffic = (struct traffic_st *)realloc(day_traffic, sizeof(struct traffic_st) * (*entry_num+1));					
	
				strncpy(day_traffic[*entry_num].ip, ip, 20);
				day_traffic[*entry_num].in = in;
				day_traffic[*entry_num].out = out;
				day_traffic[*entry_num].total = total;						
		
				(*day_in) += in;	
				(*day_out) += out;	
				(*day_total) += total;	

				(*entry_num)++;
				
				free(ip);
	
			}
	
		}
		
		
		fclose(logfile);	
	}



	
	
	return day_traffic;
}


int main(int args, char **argv)
{

	sqlite3 *openvcp_db;
	char *ipfm_dir, *sqlite_db_path, *sql_error; 
	char sql_query[1024+1];
	
  
	struct traffic_st *day_traffic;
	int month, day = 1, year;

	double day_total, day_in, day_out;
	int ipid, i=1, ret, entry_num;
	struct vsip_entry *iplist;


	
	if(args < 2)
	{
		printf("Usage: %s sqlite_db_path [ipfm_dir month year]\n", argv[0]);
		return -1;
	}


	iplist = vserver_iplist();

	sqlite_db_path = argv[1];
	ret = sqlite3_open(sqlite_db_path, &openvcp_db);

	if(ret != 0)
	{
		printf("Can't open database: (%s)", sqlite3_errmsg(openvcp_db));
		sqlite3_close(openvcp_db);
		exit(-1);
	}    
	
		
	if(args == 5)
	{
	
		ipfm_dir = argv[2];

		month = atoi(argv[3]);
		year = atoi(argv[4]);

		printf("parsing IPFM logs: %s into SQLite DB: %s\n\n", ipfm_dir, sqlite_db_path);

		    

		ret = sqlite3_exec(openvcp_db, 
			"CREATE TABLE vserver_traffic ('month', 'day', 'year', 'ip', 'in', 'out', 'total', UNIQUE(month, day, year, ip))"
									, NULL, 0, NULL);
		ret = sqlite3_exec(openvcp_db, 
			"CREATE TABLE node_traffic ('month', 'day', 'year', 'in', 'out', 'total', UNIQUE(month, day, year))"
									, NULL, 0, NULL);
		

		while(day != 32)
		{
		
			
			day_traffic = parse_day_traffic(ipfm_dir, month, day, &day_in, &day_out, &day_total, &entry_num);
			
			i = 0;
			
			while(i != entry_num)
			{
			
				ipid = 0;
				while(iplist[ipid].ip != NULL)
				{
					if(strcmp(iplist[ipid].ip, day_traffic[i].ip) == 0)
							break;
				
					ipid++;
				}
				
				if(iplist[ipid].name != NULL)
				{		
					snprintf(sql_query, 1024, 
						"INSERT OR REPLACE INTO vserver_traffic VALUES(%d, %d, %d, %u, %.0lf, %.0lf, %.0lf, '%s')",
						month, day, year, str2ip(day_traffic[i].ip), day_traffic[i].in, day_traffic[i].out, day_traffic[i].total, iplist[ipid].name);
					
					
					ret = sqlite3_exec(openvcp_db, sql_query, NULL, 0, &sql_error);

					if(ret != SQLITE_OK)
					{
						printf("SQL error (%s)", sql_error);
						sqlite3_free(sql_error);
						sqlite3_close(openvcp_db);
						exit(-1);
					}
				}
				
				i++;
			}
			
			snprintf(sql_query, 1024, 
					"INSERT OR REPLACE INTO node_traffic VALUES(%d, %d, %d, %.0lf, %.0lf, %.0lf)",
					month, day, year,
					day_in, day_out, day_total);
			

			
			ret = sqlite3_exec(openvcp_db, sql_query, NULL, 0, &sql_error);

			if(ret != SQLITE_OK)
			{
				printf("SQL error (%s)", sql_error);
				sqlite3_free(sql_error);
				sqlite3_close(openvcp_db);
				exit(-1);
			}
			
			printf("Month: %d\tDay: %d\tYear: %d\tIn: %.0lf\tOut: %.0lf\tTotal: %.0lf\n",  month, day, year, day_in, day_out, day_total);
			free(day_traffic);
			
			
			day++;
		}
		

	}
	else
	{
		i=0;
		
		while(iplist[i].ip != NULL)
		{
		
			
			if(strstr(iplist[i].ip, ":") == NULL)
			{
				snprintf(sql_query, 1024, 
						"UPDATE vserver_traffic SET name='%s' WHERE ip='%u'",
						iplist[i].name, str2ip(iplist[i].ip));
			}
			else
			{
				snprintf(sql_query, 1024, 
						"UPDATE vserver_traffic SET name='%s' WHERE ip='%s'",
						iplist[i].name, iplist[i].ip);
			
			}
		
			ret = sqlite3_exec(openvcp_db, sql_query, NULL, 0, &sql_error);
	
			if(ret != SQLITE_OK)
			{
				printf("SQL error (%s)", sql_error);
				sqlite3_free(sql_error);
				sqlite3_close(openvcp_db);
				exit(-1);
			}
			
			i++;
		}	
		
		printf("OK\n");
	
	}

	free_iplist(iplist);
	sqlite3_close(openvcp_db);	

	return 0;
}

